/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.sensor;

import java.util.Calendar;
import java.util.Date;
import nl.cloudfarming.client.geoviewer.Layer;
import nl.cloudfarming.client.geoviewer.LayerProvider;
import nl.cloudfarming.client.geoviewer.Palette;
import nl.cloudfarming.client.geoviewer.events.LayerEventKey;
import nl.cloudfarming.client.geoviewer.events.NewLayerEvent;
import nl.cloudfarming.client.geoviewer.events.RemovedLayerEvent;
import nl.cloudfarming.eventbus.BaseGuiEventProducer;
import nl.cloudfarming.eventbus.GuiEvent;
import nl.cloudfarming.eventbus.GuiEventBus;
import nl.cloudfarming.eventbus.GuiEventKey;
import nl.cloudfarming.eventbus.GuiEventListener;
import org.openide.util.Lookup;
import org.openide.util.lookup.AbstractLookup;
import org.openide.util.lookup.InstanceContent;
import org.openide.util.lookup.ServiceProvider;
import org.openide.util.lookup.ServiceProviders;

/**
 *
 * @author Timon Veenstra
 */
@ServiceProviders(value = {
    @ServiceProvider(service = LayerProvider.class),
    @ServiceProvider(service = SensorDataLayerProvider.class)})
public class GenericSensorDataLayerProvider implements SensorDataLayerProvider {

    private final InstanceContent lookupContent = new InstanceContent();
    private final Lookup lookup = new AbstractLookup(lookupContent);
    private final LayerEventProducer eventProducer = new LayerEventProducer();

    public GenericSensorDataLayerProvider() {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());

//        add(new SensorDataLayer(calendar.get(Calendar.YEAR),SensorDataLayerObject.KeyAttribute.Vi1));
//        add(new SensorDataLayer(calendar.get(Calendar.YEAR),SensorDataLayerObject.KeyAttribute.Vi2));
        add(new UnlinkedSensorDataLayer(SensorDataLayerObject.KeyAttribute.Vi1));
        add(new LinkedSensorDataLayer(SensorDataLayerObject.KeyAttribute.Vi1));

        GuiEventBus.addListener(new UpdatedSensorDataListener());

    }

    private void add(Layer layer) {
        this.lookupContent.add(layer);
    }

    @Override
    public void changePalette(Palette pallete) {
        for (Layer layer : getLookup().lookupAll(Layer.class)) {
            layer.setPalette(pallete);
        }
    }

    @Override
    public Lookup getLookup() {
        return this.lookup;
    }

    private class UpdatedSensorDataListener implements GuiEventListener<Object> {

        @Override
        public void onEvent(GuiEvent<Object> event) {
            for (Layer layer : getLookup().lookupAll(Layer.class)) {
                //
                // remove and add the unlinked layer
                //
                if (layer instanceof UnlinkedSensorDataLayer) {
                    //
                    // remove old
                    //
                    lookupContent.remove(layer);
                    eventProducer.triggerEvent(new RemovedLayerEvent(layer));
                    //
                    // add new
                    //
                    Layer newLayer = new UnlinkedSensorDataLayer(SensorDataLayerObject.KeyAttribute.Vi1);
                    eventProducer.triggerEvent(new NewLayerEvent(newLayer));
                    add(newLayer);
                }
                //
                // remove and add the linked layer when sensor data is linked
                //
                if (event.getKey().equals(SensorEventKey.SENSOR_DATA_LINKED) && layer instanceof LinkedSensorDataLayer) {
                    //
                    // remove old
                    //
                    lookupContent.remove(layer);
                    eventProducer.triggerEvent(new RemovedLayerEvent(layer));
                    //
                    // add new
                    //
                    Layer newLayer = new LinkedSensorDataLayer(SensorDataLayerObject.KeyAttribute.Vi1);
                    eventProducer.triggerEvent(new NewLayerEvent(newLayer));
                    add(newLayer);

                }
            }
        }

        @Override
        public boolean listensTo(GuiEventKey guiEventKey) {
            return SensorEventKey.SENSOR_DATA_IMPORTED.equals(guiEventKey) || SensorEventKey.SENSOR_DATA_LINKED.equals(guiEventKey);
        }

        @Override
        public String getModuleName() {
            return "sensor";
        }
    }

    private class LayerEventProducer extends BaseGuiEventProducer {

        public LayerEventProducer() {
            super(LayerEventKey.class);
        }
    }
}
